<?php
	/*
		This is a derived work.
		Derived from: 	Calibre, licensed under GPL v3
						http://calibre-ebook.com

		Changes:
		license:	 GPL v3
		copyright:	 2010, Charles Haley
	                 http://charles.haleys.org
	*/

	/*
	 * This function is required by the database. It is probably never called
	 * because it is found in an update trigger.
	 */
	function title_sort($title) {
		$title = trim($title);
		if (preg_match('/^(A|The|An)\s+/i', $title, $match)) {
			$prep = $match[1];
			$title = substr($title, strlen($prep)) . ', ' . $prep;
		}
		return trim($title);
	}

	/*
	 * map category names to their respective icons.
	 */
	$category_icon_map = array(
					'authors'   => 'user_profile.png',
					'series'    => 'series.png',
					'formats'   => 'book.png',
					'publisher' => 'publisher.png',
					'rating'    => 'rating.png',
					'news'      => 'news.png',
					'tags'      => 'tags.png',
					':custom'   => 'column.png',
					':user'     => 'drawer.png',
					'search'    => 'search.png'
			);

	function get_icon_path($key, $db) {
		global $category_icon_map;

		$prefix = 'images/';
		if ($db->fm->is_custom_field($key))
			return $prefix . $category_icon_map[':custom'];
		if ($db->fm->is_user_category($key))
			return $prefix . $category_icon_map[':user'];
		if (array_key_exists($key, $category_icon_map))
			return $prefix . $category_icon_map[$key];
		return $prefix . 'book.png';
	}

	/*
	 * Format a date using the same format strings that calire's GUI uses.
	 */
	function format_date($date, $format) {
		if (! $format)
			$format = 'dd MMM yyyy';

		/*
		 * Apparently dates in calibre's database exist in multiple forms. Some
		 * have the timezone, some don't. Some have milliseconds, some don't.
		 * Some have time, some have only date. Try several parses, in hopes
		 * that one will match
		 */
		if (!($dt = DateTime::createFromFormat('Y-m-d H:i:s O', $date)))
			if (!($dt = DateTime::createFromFormat('Y-m-d H:i:s.u O', $date)))
				if (!($dt = DateTime::createFromFormat('Y-m-d H:i:s', $date)))
					if (!($dt = DateTime::createFromFormat('Y-m-d', $date)))
						return "bad date string $date";

		/*
		 * Define some functions that will be called by the regexp matcher to
		 * replace format strings with values.
		 */
		$format_day = function($matches) use ($dt) {
			$l = strlen($matches[0]);
			if ($l == 1) return $dt->format('j');
			if ($l == 2) return $dt->format('d');
			if ($l == 3) return $dt->format('D');
			return $dt->format('l');
		};

		$format_month = function($matches) use ($dt) {
			$l = strlen($matches[0]);
			if ($l == 1) return $dt->format('n');
			if ($l == 2) return $dt->format('m');
			if ($l == 3) return $dt->format('M');
			return $dt->format('F');
		};

		$format_year = function($matches) use ($dt) {
			$l = strlen($matches[0]);
			if ($l == 2) return $dt->format('y');
			return $dt->format('Y');
		};
		$format = preg_replace_callback('/d{1,4}/', $format_day, $format);
		$format = preg_replace_callback('/M{1,4}/', $format_month, $format);
		$format = preg_replace_callback('/yyyy|yy/', $format_year, $format);
		return $format;
	}

	/*
	 * A debug print function that tried to put the output where it can later
	 * be found.
	 */
	function dprint($msg) {
		if (PHP_OS == 'Linux')
			error_log($msg . "\n");
		else
			error_log($msg . "\n", 3, 'c:/php_errors.txt');
	}
?>